program main

  use vtype_module
  use const_module, only : grav, const_set
  use ni3_module
  use ca_module
  use netcdf

  implicit none

  interface
    subroutine findfu( fn, ios, fu, mode )
      use vtype_module
      implicit none
      character(len=*), intent(in )           :: fn
      integer(i4b)    , intent(out)           :: ios, fu
      character(len=*), intent(in ), optional :: mode
    end subroutine findfu
  end interface

  integer                          :: daydiv = 24

  character(len=extstr)            :: mode

  character(len=extstr)            :: ncfn_in
  character(len=extstr)            :: ncfn_ls
  character(len=extstr)            :: ncfn_out
  integer                          :: ncid_in
  integer                          :: ncid_ls
  integer                          :: ncid_out
  character(len=extstr)            :: varname_in
  character(len=extstr)            :: varname_ls

  character(len=extstr)            :: name
!!$  character(len=extstr)            :: stdname
!!$  character(len=extstr)            :: units

  integer(i4b)                       :: NDims
  character(len=extstr), allocatable :: a_DimNames(:)

  integer                          :: imax
  integer                          :: jmax
  integer                          :: kmax
  integer                          :: tmax

  real(dp)                         :: FillValue

  real(dp)           , allocatable :: x_Lon(:)
  real(dp)           , allocatable :: y_Lat(:)
  real(dp)           , allocatable :: z_VLev(:)
  real(dp)           , allocatable :: xy_ArrB  (:,:)
  real(dp)           , allocatable :: xyz_ArrB (:,:,:)
  real(dp)           , allocatable :: xy_Arr   (:,:)
  real(dp)           , allocatable :: xyz_Arr  (:,:,:)
  real(dp)           , allocatable :: xy_DMArr (:,:)
  real(dp)           , allocatable :: xyz_DMArr(:,:,:)
  real(dp)           , allocatable :: a_Time(:)
  logical            , allocatable :: x_FlagInterpolate(:)
  real(dp)           , allocatable :: x_LSTB(:)
  real(dp)           , allocatable :: x_LST (:)


  real(dp)                         :: Time
  real(dp)                         :: Ls
  real(dp)                         :: DMTime
  real(dp)                         :: DMLs
  real(dp)                         :: PrLs

  logical                          :: ex
  integer                          :: ios

  integer                          :: DayStart
  integer                          :: DayEnd

  logical                          :: FlagLST
  real(dp)                         :: TargetLST

  integer                          :: i
  integer                          :: j
  integer                          :: k
  integer                          :: t
  integer                          :: t1
  integer                          :: t2
  integer                          :: t1s
  integer                          :: t1e


  character(extstr)                :: fn

  character(extstr)                :: ctlfn = "mkncfile.cntl"
  integer                          :: ctlfu

  namelist /input/   ncfn_in, varname_in, ncfn_ls, varname_ls
  namelist /output/  ncfn_out
  namelist /dayinfo/ DayStart, DayEnd
  namelist /pickuplst/ FlagLST, TargetLST

  ncfn_in     = "ps"
  varname_in  = "Ps"
  ncfn_ls     = "ls"
  varname_ls  = "Ls"
  ncfn_out    = "out.nc"

!      dir    = "."
!      outfn  = "./dmps.dat"
!      landersym = "vl1"
!      landersym = "vl2"


  fn = ctlfn
  inquire( file = fn, exist = ex )
  if( .not. ex ) then
    write( 6, * ) "Control file ", trim( ctlfn ), " does not exit."
    write( 6, * ) "Control file should contain namelists listed below."
    write( 6, * ) "    &input"
    write( 6, * ) "        dir       = '.'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &lander"
    write( 6, * ) "        LanderSym = 'VL1'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &output"
    write( 6, * ) "        outfn     = 'vl1_dmps.dat'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &dayinfo"
    write( 6, * ) "        DayStart  =     1"
    write( 6, * ) "        DayEnd    = 10000"
    write( 6, * ) "    &end"
    stop
  end if
  call findfu( fn, ios, ctlfu )
  if( ios /= 0 ) then
    write( 6, * ) 'STOP: ', ios
    stop
  end if
  open( ctlfu, file = fn, status = 'unknown' )
  read( ctlfu, input )
  rewind( ctlfu )
  read( ctlfu, output )
  rewind( ctlfu )
  read( ctlfu, dayinfo )
  rewind( ctlfu )
  read( ctlfu, pickuplst )
  close( ctlfu )


  call const_set( 1 )


  if ( FlagLST ) then
    write( 6, * ) 'Values at ', TargetLST, ' LST is picked up.'
  end if


  !----------------------------------------------------------
  !
  ! Preparing for ps file
  !
  mode = 'read'
  call ni3_open( ncfn_in, mode, ncid_in )
  call ni3_open( ncfn_ls, mode, ncid_ls )
  !
  !----------------------------------------------------------

  !
  ! Number of dimensions are checked.
  !
  call ni3_inq_var( ncid_in, varname_in, ndims = NDims )
  allocate( a_DimNames( NDims ) )
  call ni3_inq_vardimnames( ncid_in, varname_in, NDims, a_DimNames )

  call ni3_inq_dimlen( ncid_in, a_DimNames(1)    , imax )
  call ni3_inq_dimlen( ncid_in, a_DimNames(2)    , jmax )
  call ni3_inq_dimlen( ncid_in, a_DimNames(NDims), tmax )
  allocate( x_Lon (0:imax-1) )
  allocate( y_Lat (1:jmax  ) )
  allocate( a_Time(1:tmax  ) )
  call ni3_get_var( ncid_in, a_DimNames(1)    , x_Lon  )
  call ni3_get_var( ncid_in, a_DimNames(2)    , y_Lat  )
  call ni3_get_var( ncid_in, a_DimNames(NDims), a_Time )

  select case ( NDims )
  case ( 4 )
    call ni3_inq_dimlen( ncid_in, a_DimNames(3), kmax )
    allocate( z_VLev(1:kmax) )
    call ni3_get_var( ncid_in, a_DimNames(3), z_VLev )
  end select

  select case ( NDims )
  case ( 3 )
    allocate( xy_Arr   (0:imax-1, 1:jmax) )
    allocate( xy_DMArr (0:imax-1, 1:jmax) )
  case ( 4 )
    allocate( xyz_Arr  (0:imax-1, 1:jmax, 1:kmax) )
    allocate( xyz_DMArr(0:imax-1, 1:jmax, 1:kmax) )
  end select

  if ( FlagLST ) then
    allocate( x_FlagInterpolate(0:imax-1) )
    allocate( x_LSTB(0:imax-1) )
    allocate( x_LST (0:imax-1) )
    select case ( NDims )
    case ( 3 )
      allocate( xy_ArrB  (0:imax-1, 1:jmax) )
    case ( 4 )
      allocate( xyz_ArrB (0:imax-1, 1:jmax, 1:kmax) )
    end select
  end if


  !----------------------------------------------------------
  !
  ! Preparing for output file
  !
  mode = 'new'
  call ni3_open( ncfn_out, mode, ncid_out )

  name = a_DimNames(1)
  call ni3_set_dim( ncid_out, name, NF90_REAL, x_Lon )
  call ni3_cp_atts( ncid_in, ncid_out, name )

  name = a_DimNames(2)
  call ni3_set_dim( ncid_out, name, NF90_REAL, y_Lat )
  call ni3_cp_atts( ncid_in, ncid_out, name )

  select case ( NDims )
  case ( 4 )
    name = a_DimNames(3)
    call ni3_set_dim( ncid_out, name, NF90_REAL, z_VLev )
    call ni3_cp_atts( ncid_in, ncid_out, name )
  end select

!!$  name    = "time"
!!$  stdname = "time"
!!$  units   = "days since 0000-01-01"
!!$  call ni3_def_dim( ncid_out, name, NF90_REAL, NF90_UNLIMITED, stdname = stdname, units  = units )
!!$  call ni3_cp_atts( ncid_ice, ncid_out, name )

  name = varname_ls
  call ni3_def_dim( ncid_out, name, NF90_REAL, NF90_UNLIMITED )
  call ni3_cp_atts( ncid_ls, ncid_out, name )

!!$  a_DimNames( 1 ) = "lon"
!!$  a_DimNames( 2 ) = "lat"
!!$  dimname( 3 ) = "time"
  a_DimNames( NDims ) = varname_ls

  name = varname_in
  call ni3_def_var( ncid_out, name, NF90_REAL, ndims, a_DimNames )
  call ni3_cp_atts( ncid_in, ncid_out, name )

  if ( ni3_chk_att( ncid_in, varname_in, "missing_value" ) ) then
    call ni3_get_att( ncid_in , varname_in, "missing_value", FillValue )
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  else if ( ni3_chk_att( ncid_in, varname_in, "_FillValue" ) ) then
    call ni3_get_att( ncid_in , varname_in, "_FillValue"   , FillValue )
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  else
    FillValue = -999.0
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  end if
  !
  !----------------------------------------------------------


  write( 6, * ) "---"

  PrLs     = -1.0d100

  t1s = max( DayStart, 1 )
  t1e = min( DayEnd, (tmax-1)/daydiv )
!!$  time_loop: do t1 = 1, (tmax-1)/daydiv
  time_loop: do t1 = t1s, t1e

    print *, "Day ", t1

    select case ( NDims )
    case ( 3 )
      xy_DMArr =  0.0d0
    case ( 4 )
      xyz_DMArr =  0.0d0
    end select
    DMTime   =  0.0d0
    DMLs     =  0.0d0
!!$    PrLs     = -1.0d100

    if ( FlagLST ) then
      x_FlagInterpolate = .false.

      if ( t1 == t1s ) then
        t2 = 0
        t = ( t1 - 1 ) * daydiv + t2 + 1
        !
        select case ( NDims )
        case ( 3 )
          call ni3_get_varss( ncid_in, varname_in, t, xy_Arr )
          xy_ArrB = xy_Arr
        case ( 4 )
          call ni3_get_varss( ncid_in, varname_in, t, xyz_Arr )
          xyz_ArrB = xyz_Arr
        end select
        !
        x_LST = mod( a_Time(t) + 24.0d0 * x_Lon / 360.0d0, 24.0d0 )
        x_LSTB = x_LST
      end if
    end if

    do t2 = 1, daydiv
      ! A lat '+1' means that an initial condition is stored in a file.
      t = ( t1 - 1 ) * daydiv + t2 + 1

      !
      ! Read data
      !
      select case ( NDims )
      case ( 3 )
        call ni3_get_varss( ncid_in, varname_in, t, xy_Arr )
      case ( 4 )
        call ni3_get_varss( ncid_in, varname_in, t, xyz_Arr )
      end select


      if ( FlagLST ) then
        !
        ! Calculation of local solar time
        !
        x_LST = mod( a_Time(t) + 24.0d0 * x_Lon / 360.0d0, 24.0d0 )

        select case ( NDims )
        case ( 3 )
          do i = 0, imax-1
            if ( ( x_LSTB(i) <= TargetLST ) .and. &
              &  ( TargetLST <= x_LST(i)  ) ) then
              x_FlagInterpolate(i) = .true.
              do j = 1, jmax
                xy_DMArr(i,j) =                      &
                  &   ( xy_Arr(i,j) - xy_ArrB(i,j) ) &
                  & / ( x_LST(i)    - x_LSTB(i)    ) &
                  & * ( TargetLST   - x_LSTB(i)    ) &
                  & + xy_ArrB(i,j)
              end do
            end if
          end do
        case ( 4 )
          do i = 0, imax-1
            if ( ( x_LSTB(i) <= TargetLST ) .and. &
              &  ( TargetLST <= x_LST(i)  ) ) then
              x_FlagInterpolate(i) = .true.
              do k = 1, kmax
                do j = 1, jmax
                  if ( ( xyz_Arr (i,j,k) /= FillValue ) .and. &
                    &  ( xyz_ArrB(i,j,k) /= FillValue ) ) then
                    xyz_DMArr(i,j,k) =                         &
                      &   ( xyz_Arr(i,j,k) - xyz_ArrB(i,j,k) ) &
                      & / ( x_LST(i)       - x_LSTB(i)       ) &
                      & * ( TargetLST      - x_LSTB(i)       ) &
                      & + xyz_ArrB(i,j,k)
                  else
                    xyz_DMArr(i,j,k) = FillValue
                  end if
                end do
              end do
            end if
          end do
        end select

      else

        select case ( NDims )
        case ( 3 )
          xy_DMArr = xy_DMArr + xy_Arr
        case ( 4 )
!!$        xyz_DMArr = xyz_DMArr + xyz_Arr
          call ca_add( imax, jmax, kmax, xyz_Arr, xyz_DMArr, FillValue )
        end select

      end if

      if ( FlagLST ) then
        x_LSTB = x_LST
        select case ( NDims )
        case ( 3 )
          xy_ArrB  = xy_Arr
        case ( 4 )
          xyz_ArrB = xyz_Arr
        end select
      end if

      !
      ! Read Ls data
      !
      call ni3_get_varss( ncid_ls, varname_ls, t, Ls )

      time = a_Time(t)

!!$      if( Ls < PrLs ) Ls = Ls + 360.0d0
      if( Ls < PrLs )then
        do
          Ls = Ls + 360.0d0
          if( Ls > PrLs ) exit
        end do
      end if


      DMTime = DMTime + time
      DMLs   = DMLs   + Ls
      PrLs   = Ls

    end do ! t2 loop

    ! Check of missing data in an array in the case of LST interpolation
    !
    if ( FlagLST ) then
      if ( .not. all( x_FlagInterpolate ) ) then
        write( 6, * ) 'Not all of array elements are not calculated.'
      end if
    end if

    if ( .not. FlagLST ) then
      select case ( NDims )
      case ( 3 )
        xy_DMArr  = xy_DMArr  / dble( daydiv )
      case ( 4 )
        call ca_div( imax, jmax, kmax, xyz_DMArr, dble( daydiv ), FillValue )
      end select
    end if
    DMTime   = DMTime   / dble( daydiv )
    DMLs     = DMLs     / dble( daydiv )

!!$    DMLs     = mod( DMLs, 360.0d0 )


!!$    if( ( t1 >= DayStart ) .and. ( t1 <= DayEnd ) ) then
!!$      call ni3_put_varss( ncid_out, "time", t1 - DayStart + 1, DMTime )
      call ni3_put_varss( ncid_out, varname_ls, t1 - DayStart + 1, DMLs     )
      select case ( NDims )
      case ( 3 )
        call ni3_put_varss( ncid_out, varname_in, t1 - DayStart + 1, xy_DMArr )
      case ( 4 )
        call ni3_put_varss( ncid_out, varname_in, t1 - DayStart + 1, xyz_DMArr )
      end select
!!$    else
!!$      print *, " --- No output"
!!$    end if


  end do time_loop


  call ni3_close( ncid_in  )
  call ni3_close( ncid_ls  )

  call ni3_close( ncid_out )

end program main
